@@ -45,6 +45,9 @@
 
 ICurrentProcess* System();
 
+#define WIDTH   1024
+#define HEIGHT  768
+
 /****************************************************************************/
 /*              Keyboard and Mouse                                          */
 /****************************************************************************/
@@ -70,12 +73,23 @@ extern "C"
     };
 
     int inputSemaphoreIndex = 0;/* if non-zero the event semaphore index */
+
+    /*** Variables -- Imported from Virtual Machine ***/
+    extern int interruptPending;
+    extern int interruptCheckCounter;
+    extern int interruptKeycode;
+    extern int fullScreenFlag;
+    extern int deferDisplayUpdates; /* Is the Interpreter doing defered updates for us?! */
+
+    int synchronizedSignalSemaphoreWithIndex(int semaIndex);
 }
 
 class EventQueue
 {
     static const int KEYBUF_SIZE = 64;
     static const int MAX_EVENT_BUFFER = 1024;
+    static const int AUTO_REPEAT_RATE = 4;
+    static const int AUTO_REPEAT_DELAY = 20;
 
     IMonitor* monitor;
 
@@ -89,6 +103,7 @@ class EventQueue
     int modifiers;
     bool caps;
     bool numlock;
+    int repeat;
 
     int width;
     int height;
@@ -188,7 +203,7 @@ class EventQueue
         {
         case KEYBOARD_HOME:
             return 1;
-        case KEYBOARD_ENTER:
+        case KEYPAD_ENTER:
             return 3;
         case KEYBOARD_END:
             return 4;
@@ -202,7 +217,7 @@ class EventQueue
             return 11;
         case KEYBOARD_PAGEDOWN:
             return 12;
-        case KEYBOARD_RETURN:
+        case KEYBOARD_ENTER:
             return 13;
         case KEYBOARD_LEFTALT:
             return 17;
@@ -483,8 +498,9 @@ public:
         modifiers(0),
         caps(false),
         numlock(true),
-        width(1024),
-        height(768),
+        repeat(0),
+        width(WIDTH),
+        height(HEIGHT),
         button(0),
         x(width / 2),
         y(height / 2)
@@ -544,9 +560,10 @@ public:
         y = this->y;
     }
 
-    void keyEvent(u8* data, long size)
+    bool keyEvent(u8* data, long size)
     {
         Synchronized<IMonitor*> method(monitor);
+        bool notify(false);
 
         using namespace UsageID;
 
@@ -569,6 +586,7 @@ public:
             ASSERT(0 <= key);
             bits &= ~(1u << key);
             keyDown(key + KEYBOARD_LEFTCONTROL);
+            notify = true;
         }
 
         bits = (*from ^ *to) & *to;
@@ -578,6 +596,7 @@ public:
             ASSERT(0 <= key);
             bits &= ~(1u << key);
             keyUp(key + KEYBOARD_LEFTCONTROL);
+            notify = true;
         }
 
         u8 mod(*to | (*to >> 4));
@@ -588,33 +607,48 @@ public:
                     ((mod & (1<<(0x0f & KEYBOARD_RIGHTSHIFT))) ? ShiftKeyBit : 0) |
                     ((mod & (1<<(0x0f & KEYBOARD_RIGHTALT))) ? OptionKeyBit : 0);
 
+        bool repeated(false);
         *from++ = *to++;
         do
         {
             if (*from < *to)
             {
                 keyUp(*from);
+                notify = true;
                 ++from;
             }
             else if (*to < *from)
             {
                 keyDown(*to);
+                notify = true;
                 ++to;
             }
             else
             {
-                // XXX auto
+                repeated = true;
+                if (++repeat == AUTO_REPEAT_DELAY)
+                {
+                    keyDown(*to);   // Auto repeat
+                    repeat = AUTO_REPEAT_DELAY - AUTO_REPEAT_RATE;
+                }
                 ++from;
                 ++to;
             }
         } while (*to != *from || *to != 255);
+        if (!repeated)
+        {
+            repeat = 0;
+        }
 
         memmove(key, next, 9);
+
+        return notify;
     }
 
-    void mouseEvent(u8* data, long size)
+    bool mouseEvent(u8* data, long size)
     {
         Synchronized<IMonitor*> method(monitor);
+        bool notify(false);
 
         using namespace UsageID;
 
@@ -625,39 +659,59 @@ public:
             {
                 keyDown(KEYBOARD_PAGEUP);
                 keyUp(KEYBOARD_PAGEUP);
+                notify = true;
             }
             else if (0 < z)
             {
                 keyDown(KEYBOARD_PAGEDOWN);
                 keyUp(KEYBOARD_PAGEDOWN);
+                notify = true;
             }
         }
 
-        if (size < 3 || !(data[0] ^ button) && !data[1] && !data[2])
+        if (3 <= size && ((data[0] ^ button) || data[1] || data[2]))
         {
-            return;
-        }
-        button = data[0];
-        x += (s8) data[1];
-        y -= (s8) data[2];
-        if (x < 0)
-        {
-            x = 0;
-        }
-        if (width <= x)
-        {
-            x = width - 1;
-        }
-        if (y < 0)
-        {
-            y = 0;
-        }
-        if (height <= y)
-        {
-            y = height - 1;
+            button = data[0];
+            x += (s8) data[1];
+            y -= (s8) data[2];
+            if (x < 0)
+            {
+                x = 0;
+            }
+            if (width <= x)
+            {
+                x = width - 1;
+            }
+            if (y < 0)
+            {
+                y = 0;
+            }
+            if (height <= y)
+            {
+                y = height - 1;
+            }
+
+            mouse(button, x, y);
+            notify = true;
         }
 
-        mouse(button, x, y);
+        return notify;
+    }
+
+    bool wait(long long timeout)
+    {
+        Synchronized<IMonitor*> method(monitor);
+
+        if (eventRing.getUsed() == 0)
+        {
+            return monitor->wait(timeout);
+        }
+        return false;
+    }
+
+    void notify()
+    {
+        monitor->notifyAll();
     }
 };
 
@@ -672,6 +726,17 @@ Handle<ICursor> cursor;
 int bpp;
 void* framebufferPtr;
 
+/*** Synchronization functions ***/
+
+int synchronizedSignalSemaphoreWithIndex(int semaIndex)
+{
+    /* do our job */
+    int result = signalSemaphoreWithIndex(semaIndex);
+    /* wake up interpret() if sleeping */
+    eventQueue.notify();
+    return result;
+}
+
 /* User input recording I:
    In general, either set of input function can be supported,
    depending on the platform. This (first) set is state based
@@ -808,11 +873,10 @@ int ioSetCursorWithMask(int cursorBitsIndex, int cursorMaskIndex, int offsetX, i
 
 int ioRelinquishProcessorForMicroseconds(int microSeconds)
 {
-    Handle<ICurrentThread> current(System()->currentThread());
-    current->sleep(microSeconds * 10);
-
-    // aioPoll(microSeconds);
-    return 0;
+    /* wake us up if something happens */
+    eventQueue.wait(microSeconds * 10LL);
+    interruptCheckCounter = 0;  // for smooth eToy animation, etc.
+    return microSeconds;
 }
 
 int ioForceDisplayUpdate(void)
@@ -823,7 +887,7 @@ int ioForceDisplayUpdate(void)
 int ioScreenSize(void)
 {
     /* return the screen size as two positive 16-bit integers packed into a 32-bit integer */
-    int w = 1024, h = 768;
+    int w = WIDTH, h = HEIGHT;
 
     return (w << 16) | (h & 0xFFFF);  /* w is high 16 bits; h is low 16 bits */
 }
@@ -914,7 +978,7 @@ int ioHasDisplayDepth(int depth)
 
 int ioSetDisplayMode(int width, int height, int depth, int fullscreenFlag)
 {
-    if (width == 1024 && height == 768 && depth == 32)
+    if (width == WIDTH && height == HEIGHT && depth == 32)
     {
         return 1;   // accept
     }
@@ -949,7 +1013,7 @@ void* inputProcess(void* param)
                                  ICurrentProcess::PROT_READ | ICurrentProcess::PROT_WRITE,
                                  ICurrentProcess::MAP_SHARED,
                                  framebuffer, 0);
-    bpp = 8 * (size / (1024 * 768));
+    bpp = 8 * (size / (WIDTH * HEIGHT));
 
     cursor = root->lookup("device/cursor");
     cursor->setPosition(x, y);
@@ -959,16 +1023,21 @@ void* inputProcess(void* param)
     {
         u8 buffer[8];
         long count;
+        bool notify(false);
 
         count = keyboard->read(buffer, 8);
-        eventQueue.keyEvent(buffer, count);
+        notify |= eventQueue.keyEvent(buffer, count);
 
         count = mouse->read(buffer, 4);
-        eventQueue.mouseEvent(buffer, count);
+        notify |= eventQueue.mouseEvent(buffer, count);
 
         eventQueue.getMousePoint(x, y);
         cursor->setPosition(x, y);
 
+        if (notify)
+        {
+            eventQueue.notify();
+        }
         currentThread->sleep(10000000 / 60);
     }
 
